Package: hmde
Title: Hierarchical Methods for Differential Equations
Version: 1.4.0
Authors@R: 
    c(
  person(given = "Daniel", family = "Falster", role = c("aut", "ctb"), email = "daniel.falster@unsw.edu.au", comment = c(ORCID = "0000-0002-9814-092X")),
  person(given = "Tess", family = "O'Brien", role = c("aut", "cre", "cph"), email = "tess_obrien@fastmail.com", comment = c(ORCID = "0009-0000-5821-8801")),
  person(given = "Fonti", family = "Kar", role = c("ctb"), email = "f.kar@unsw.edu.au", comment = c(ORCID = "0000-0002-2760-3974")),
  person(given = "David", family=  "Warton", role = c("aut", "ctb"), email = "david.warton@unsw.edu.au", comment = c(ORCID = "0000-0002-1642-628X"))
    )
Description: Wrapper for 'Stan' that offers a number of in-built models to implement a hierarchical Bayesian longitudinal model for repeat observation data. Model choice selects the differential equation that is fit to the observations. Single and multi-individual models are available. O'Brien et al. (2024) <doi:10.1111/2041-210X.14463>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Biarch: true
Depends: R (>= 4.1.0)
Imports: methods, dplyr, ggplot2, Rcpp (>= 0.12.19), RcppParallel (>=
        5.0.1), rlang, rstan (>= 2.18.1), rstantools (>= 2.3.1.1),
        tibble, cowplot, knitr, purrr (>= 1.2.0)
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.19), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.18.1), StanHeaders (>=
        2.18.0)
SystemRequirements: GNU make
Suggests: rmarkdown, testthat (>= 3.0.0), withr, mnormt, here,
        patchwork, deSolve, mixtools, MASS
VignetteBuilder: knitr
Config/testthat/edition: 3
LazyData: true
URL: https://traitecoevo.github.io/hmde/
BugReports: https://github.com/traitecoevo/hmde/issues
NeedsCompilation: yes
Packaged: 2026-05-04 04:07:50 UTC; tess
Author: Daniel Falster [aut, ctb] (ORCID:
    <https://orcid.org/0000-0002-9814-092X>),
  Tess O'Brien [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0000-5821-8801>),
  Fonti Kar [ctb] (ORCID: <https://orcid.org/0000-0002-2760-3974>),
  David Warton [aut, ctb] (ORCID:
    <https://orcid.org/0000-0002-1642-628X>)
Maintainer: Tess O'Brien <tess_obrien@fastmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 04:50:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-04 06:53:43 UTC; unix
Archs: hmde.so.dSYM
