# Generated by roxygen2: do not edit by hand

export("error_ests<-")
export("fit_summary<-")
export("individual_ests<-")
export("measurement_ests<-")
export("method<-")
export("model_level<-")
export("model_name<-")
export("obs_data<-")
export("par_names<-")
export("population_ests<-")
export("prior_pars<-")
export("runtime<-")
export(error_ests)
export(fit_summary)
export(hmde_affine_de)
export(hmde_canham_de)
export(hmde_const_de)
export(hmde_data_template)
export(hmde_estimates)
export(hmde_extract_Rhat)
export(hmde_model)
export(hmde_model_des)
export(hmde_model_names)
export(hmde_model_pars)
export(hmde_plot_Rhat_hist)
export(hmde_plot_de_pieces)
export(hmde_plot_obs_est_inds)
export(hmde_run)
export(hmde_vb_de)
export(individual_ests)
export(measurement_ests)
export(method)
export(model_level)
export(model_name)
export(obs_data)
export(par_names)
export(population_ests)
export(prior_pars)
export(runtime)
exportClasses(hmde_data_template)
exportClasses(hmde_estimates)
exportMethods("error_ests<-")
exportMethods("fit_summary<-")
exportMethods("measurement_ests<-")
exportMethods("method<-")
exportMethods("model_level<-")
exportMethods("model_name<-")
exportMethods("obs_data<-")
exportMethods("par_names<-")
exportMethods("population_ests<-")
exportMethods("prior_pars<-")
exportMethods("runtime<-")
exportMethods(error_ests)
exportMethods(fit_summary)
exportMethods(individual_ests)
exportMethods(measurement_ests)
exportMethods(method)
exportMethods(model_level)
exportMethods(model_name)
exportMethods(obs_data)
exportMethods(par_names)
exportMethods(plot)
exportMethods(population_ests)
exportMethods(print)
exportMethods(prior_pars)
exportMethods(runtime)
exportMethods(show)
exportMethods(summary)
import(RcppParallel)
import(cowplot)
import(dplyr)
import(ggplot2)
import(knitr)
import(methods)
import(rlang)
import(rstan)
import(rstantools)
import(tibble)
importFrom(Rcpp,loadModule)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(cowplot,plot_grid)
importFrom(knitr,kable)
importFrom(purrr,list_modify)
importFrom(purrr,map)
importFrom(rstan,Rhat)
importFrom(rstan,get_elapsed_time)
importFrom(rstan,sampling)
importFrom(rstantools,rstan_config)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(utils,head)
useDynLib(hmde, .registration = TRUE)
