---
title: "Case study 2: von Bertalanffy growth with lizard size data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{von-bertalanffy}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
bibliography: vignette.bib
editor_options: 
  chunk_output_type: console
---
  
```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options(rmarkdown.html_vignette.check_title = FALSE)
```

### Load dependencies

```{r setup}
# remotes::install_github("traitecoevo/hmde")
# install.packages(c("dplyr", "ggplot2"))

library(hmde)
library(dplyr)
library(ggplot2)
```

## Overview
Our second demo introduces size-dependent growth based on the von Bertalanffy function
\begin{equation}\label{eqn_vb_DE}
f(Y(t); S_{max}, \beta) = \beta(S_{max} - Y(t)),
\end{equation}
where $S_{max}$ is the asymptotic maximum size and $\beta$ controls the growth rate. We have implemented the analytic solution
\begin{equation}\label{eqn_vb_soln}
Y(t) = S_{max} + (Y(0) - S_{max}) \exp(-t\beta)
\end{equation}
which is independent of age at the starting size $Y(0)$ and instead uses the first size as the initial condition. The key behaviour of the von Bertalanffy model is a high growth rate at small sizes that declines linearly as the size approaches $S_{max}$. This manifests as growth slowing as a creature matures with a hard finite limit on the eventual size. We restrict $\beta$ and $S_{max}$ to be positive. As a result the growth rate is non-negative. 

### Priors
The default priors for the constant top-level parameters in the single individual model are
$$S_{max} \sim \log\mathcal{N}(\log(\max(y_{obs})), 2),$$
$$\beta \sim \log\mathcal{N}(0, 2),$$
$$0 <\sigma_e \sim Cauchy(0, 2).$$

For the multi-individual model the prior structure and default parameters are
$$\mu_{S_{max}} \sim \mathcal{N}(\log(\max(y_{obs})), 2),$$
$$0 < \sigma_{S_{max}} \sim Cauchy(0, 2),$$
$$\mu_{\beta} \sim \mathcal{N}(0, 2),$$
$$0 < \sigma_{\beta} \sim Cauchy(0, 2),$$
$$0 < \sigma_{e} \sim Cauchy(0, 2).$$
The max size parameter priors are always centred at the (transformed) maximum observed size. This is not changeable, but the standard deviation is. To see the name for the prior parameter run `hmde_model`. For example in the following we want to change the prior for $S_{max}$ standard deviation (`ind_max_size`) in the individual model:
```{r}
prior_pars(hmde_model("vb_single_ind"))
#prior_pars_ind_max_size_sd_only is the argument name for the prior parameter
```

### Visualise model
In the following code we plot an example of the growth function and the solution to get a feel for the behaviour.
```{r}
#Analytic solution in function form
solution <- function(t, pars = list(y_0, beta, S_max)){
  return(
    pars$S_max + (y_0 - pars$S_max)*exp(-t * pars$beta)
  )
}

#Parameters
beta <- 0.35 #Growth rate
y_0 <- 1 #Starting size
S_max <- 20 #Asymptotic max size
time <- c(0,30) 
pars_list <- list(y_0 = y_0,
                  beta = beta,
                  S_max = S_max)
y_final <- solution(time[2], pars_list)

#Plot of growth function
ggplot() +
  xlim(y_0, y_final) +
  ylim(0, beta*(S_max-y_0)*1.1) +
  labs(x = "Y(t)", y = "f", title = "von Berralanffy growth") +
  theme_classic() +
  theme(axis.text=element_text(size=16),
        axis.title=element_text(size=18,face="bold")) +
  geom_function(fun=hmde_model_des("vb_single_ind"), 
                args=list(pars = list(S_max, beta)),
                colour="green4", linewidth=1,
                xlim=c(y_0, y_final))

#Size over time
ggplot() +
  geom_function(fun=solution, 
                args=list(pars = pars_list),
                colour="green4", linewidth=1,
                xlim=c(time)) +
  xlim(time) +
  ylim(0, y_final*1.05) +
  labs(x = "Time", y = "Y(t)", title = "von Bertalanffy size") +
  theme_classic() +
  theme(axis.text=element_text(size=16),
        axis.title=element_text(size=18,face="bold"))

```

The von Bertalanffy model is commonly used in fishery management [@flinn2021trends], but has also been used in reptile studies such as @edmonds2021growing and @zhao2020age.

## Lizard size data
Our data is sourced from @Kar2023_LizardMass which measured mass and snout-vent-length (SVL) of delicate skinks -- \textit{Lampropholis delicata} -- under experimental conditions to examine the effect of temperature on development. We are going to use the SVL metric for size.

We took a simple random sample without replacement of 50 individuals with at least 5 observations each. The von Bertalanffy model can be fit to shorter observation lengths, but fewer than 3 observations is not advised as there are two growth parameters per individual. 

## Implementation
The workflow for the second example is the same as the first, with the change in model name and data object.
```{r}
set.seed(2026)
lizard_vb_fit <- hmde_data_template("vb_multi_ind",
                                    obs_data = Lizard_Size_Data ) |>
  hmde_run(chains = 4, cores = 1, iter = 2000)

lizard_estimates <- hmde_estimates(fit = lizard_vb_fit,                                           obs_data = Lizard_Size_Data)
```

As before, we can compare the observed sizes over time to those predicted by the model.
```{r}
measurement_data_transformed <- measurement_ests(lizard_estimates) %>%
  group_by(ind_id) %>%
  mutate(
    delta_y_obs = y_obs - lag(y_obs),
    obs_interval = time - lag(time),
    obs_growth_rate = delta_y_obs/obs_interval,
    delta_y_est = y_hat - lag(y_hat),
    est_growth_rate = delta_y_est/obs_interval
  ) %>%
  ungroup()


#Distributions of estimated growth and size
hist(measurement_data_transformed$y_hat, 
     main = "Estimated size distribution",
     xlab = "Size (cm)")
hist(measurement_data_transformed$delta_y_est, 
     main = "Estimated growth increments",
     xlab = "Growth increment (cm)")
hist(measurement_data_transformed$est_growth_rate, 
     main = "Estimated annualised growth rate distribution",
     xlab = "Growth rate (cm/yr)")

#Quantitative R^2
cor(measurement_data_transformed$y_obs, measurement_data_transformed$y_hat)^2
r_sq_est <- cor(measurement_data_transformed$y_obs,
                measurement_data_transformed$y_hat)^2
r_sq <- paste0("R^2 = ", 
               signif(r_sq_est,
                      digits = 3))

obs_scatter <- ggplot(data = measurement_data_transformed,
       aes(x = y_obs, y = y_hat)) +
  geom_point(shape = 16, size = 1, colour = "green4") +
  xlab("Y obs.") +
  ylab("Y est.") +
  geom_abline(slope = 1, linetype = "dashed") +
  annotate("text", x = 25, y = 18, 
           label = r_sq) +
  theme_classic()

#Plots of size over time for a sample of 5 individuals
obs_est_ind <- hmde_plot_obs_est_inds(estimates = lizard_estimates,
                                      n_ind_to_plot = 5) +
  theme(legend.position = "inside",
        legend.position.inside = c(0.8, 0.2))
```

We have two parameters at the individual level and are interested in both their separate distributions, and if we see evidence of a relationship between them. We can also use the individual parameter estimates and estimated sizes to plot the growth function pieces.
```{r}
#1-dimensional parameter distributions
s_max_hist <- ggplot(individual_ests(lizard_estimates), 
       aes(ind_max_size_mean)) +
  geom_histogram(bins = 10,
                 colour = "black",
                 fill = "lightblue") +
  labs(x="S_max estimate") +
  theme_classic()

beta_hist <- ggplot(individual_ests(lizard_estimates), 
       aes(ind_growth_rate_mean)) +
  geom_histogram(bins = 10,
                 colour = "black",
                 fill = "lightblue") +
  labs(x="beta estimate") +
  theme_classic()

#2-dimensional parameter distribution
par_scatter <- ggplot(data = individual_ests(lizard_estimates), 
       aes(x = ind_max_size_mean, y = ind_growth_rate_mean)) +
  geom_point(shape = 16, size = 1, colour = "green4") +
  xlab("Individual max sizes (mm)") +
  ylab("Individual growth rate parameters") +
  theme_classic()

#Correlation of parameters
cor(individual_ests(lizard_estimates)$ind_max_size_mean,
    individual_ests(lizard_estimates)$ind_growth_rate_mean,
    method = "spearman")

#Plot function pieces over estimated sizes.
de_pieces <- hmde_plot_de_pieces(lizard_estimates)
```

At the hyper-parameter level for the whole population we have centre and spread parameters for the log-normal distributions of $S_{max}$ and $\beta$. As before, we can look at these as species-level features.
```{r}
pars_CI_names <- c(
  "mean log max size",
  "mean max size in mm",
  "log max size standard deviation",
  "mean log growth par",
  "mean growth par mm/yr",
  "log growth par standard deviation"
)

#Vector that picks out which pars to be exponentiated
exp_vec <- c(FALSE, TRUE, FALSE, 
             FALSE, TRUE, FALSE)

#Print mean estimates and CIs
for(i in 1:nrow(population_ests(lizard_estimates))){
  if(!exp_vec[i]){
    population_ests(lizard_estimates)$mean[i] 
    print(paste0("95% CI for ", 
                 pars_CI_names[i],
                 ": (",
                 population_ests(lizard_estimates)$CI_lower[i],
                 ", ",
                 population_ests(lizard_estimates)$CI_upper[i],
                 ")"))
  } else {
    exp(population_ests(lizard_estimates)$mean[i])
    print(paste0("95% CI for ",
                 pars_CI_names[i], 
                 ": (",
                 exp(population_ests(lizard_estimates)$CI_lower[i]),
                 ", ",
                 exp(population_ests(lizard_estimates)$CI_upper[i]),
                 ")"))
  }
}
```

## References

